@@ -21,24 +21,26 @@ import json as JSON
 import jsonpath_ng.ext as jsonpath
 
 # Misc import
-from typing import Union
-import os.path
-from pathlib2 import Path
+from typing import Union, Any
 
 # Other modules import
 from robust_json.errors import (
     JSONFileError,
     JSONPathError,
-    JSONObjectError,
     JSONStrictModeError,
     IncorrectFunctionParameterTypeError,
 )
 from robust_json.__internal_utils import service
 
-#! Review documentation!!!
-
 
 class JsonFileParser:
+    """
+    This class provides functionality for working with JSON files.
+
+    For more information please visit:
+    https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#file-module-overview
+    """
+
     def __init__(self, path):
         self.__path = path
         self.__file_formats = [".json", ".txt"]
@@ -55,14 +57,14 @@ class JsonFileParser:
     @property
     def file_formats(self):
         """
-        All file extensions supported by this module at the moment.
+        All supported file extensions.
         """
         return self.__file_formats
 
     @property
     def path(self):
         """
-        Source file path
+        Path to file with JSON.
         """
         return self.__path
 
@@ -77,35 +79,38 @@ class JsonFileParser:
         """
         Extract all JSON from source file.
 
-        This function will read the source file and return JSON from it
+        This function reads the source file and returns JSON object from it
         as a Python dictionary.
 
-        This function is automatically called when class is
-        initialized.
-        """
+        This function is automatically called when class instance is
+        created.
 
+        For more information please visit: https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#file-module-methods-and-properties
+        """
         file = open(self.__path, "r")
         cont = file.read()
         file.close()
 
         self.active_json = JSON.loads(
             cont
-        )  # * May deprecate this line due to the fact that in __init__ uts return value is assigned to self.active_json (double assignment)
+        )  # * May deprecate this line due to the fact that in __init__ its return value is assigned to self.active_json (double assignment)
         return JSON.loads(cont)
 
-    def get_key_value(self, json_path: str) -> Union[any, dict]:
+    def get_key_value(self, json_path: str) -> Any:
         """
-        Retrieve specific value from JSON.
+        Retrieve specific key:value pair from JSON.
 
-        This function will fetch value from JSON object according to provided path.
+        This function fetches key:value pair from JSON object according to provided path
+        and returns only the value.
 
         Parameters: `json_path : str` specifies JSON property path (e.g. field1.field2.[...].fieldn).
 
-        This function will raise a `IncorrectFunctionParameterTypeError` if
+        This function raises an `IncorrectFunctionParameterTypeError` if
         `json_path` parameter has an incorrect type.
-        This function will raise a `JSONPathError` if given path is not valid.
+        This function raises a `JSONPathError` if JSON path is not valid.
+        This function raises any additional exceptions if occurred.
 
-        Example:
+        Examples:
 
         Retrieving key:value pair from a simple object:
 
@@ -126,10 +131,10 @@ class JsonFileParser:
         # arr_val = 2
 
         For more information about this method please visit:
-        <LINK_TO_DOCUMENTATION_HERE>
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#file-module-methods-and-properties
         """
-        # TODO Add link to an appropriate README section from GitHub.
 
+        # Checkint parameter type
         if type(json_path) != str:
             raise IncorrectFunctionParameterTypeError(
                 "json_path", "str", type(json_path).__name__
@@ -137,45 +142,45 @@ class JsonFileParser:
 
         json_content = self.active_json
 
+        # Verifying JSON path
         if not self.__service.check_json_path(json_path, json_content):
             raise JSONPathError(f"Path `{json_path}` is not valid.")
 
-        js_expr = jsonpath.parse(json_path)
+        js_expr = jsonpath.parse(json_path)  # Parsing JSON
 
-        res = [item.value for item in js_expr.find(json_content)]
+        res = [
+            item.value for item in js_expr.find(json_content)
+        ]  # Filling an array with all matches
 
         if len(res) == 1:
             return res[0]
         return res
 
     def append(
-        self, json_path: str, append_value: any, append_at_end: bool = False
-    ) -> None:
+        self, json_path: str, append_value: Any, append_at_end: bool = False
+    ) -> dict:
         """
         Append new value to an existing JSON object.
 
-        This function will take given value and add it to an existing JSON object.
+        This function takes value and adds it to the JSON object.
 
-        Parameters: `json_path : str` specifies JSON property path, where the given
+        Parameters: `json_path : str` specifies JSON property path where the given
         value needs to be added. If there is a need to append a value
-        to the root object, this parameter needs to be equal to `$`. `append_value : any`
+        to the root of the object, this parameter needs to be equal to `$`. `append_value : Any`
         specifies the value that will be appended to the object. `append_at_end : bool`
-        specifies if given value will be appended to the end of a JSON array or it
-        will be added into each object of this array. While this parameter has no effect
-        if value is being appended to a JSON object (dictionary), it is neccessary if
-        value is being appended to a JSON array (list). If value is being appended to an array of elements,
-        other than arrays or objects (integers, strings, etc.), setting this parameter to `False` will not
-        result in any changes made to array. It needs to be set to `True` for this operation to complete.
+        controls the behaviour of this function regarding JSON arrays of objects (structures like this: [{}, {}, {}, ...])
+        and general arrays (structures like this: [a, b, c, ...]). It has no influence on other structures. If set to False,
+        function will try to add value to each object of an array. If set to True, function will try to append
+        value at the end of an array. (see examples below).
 
 
-        This function will return a Python dictionary with updated content.
+        This function returns a Python dictionary with updated content.
 
-        This function will raise a `FunctionParameterTypeError` exception if either of given parameters
-        has an incorrect type.
-        This function will raise a `ValueError` exception if `append_value` parameter is empty (i.e
-        it's equal to an empty string, an empty array or empty dictionary).
-        This function will raise a `JSONPathError` exception if given path is not valid.
-        This function will raise any additional exceptions if occurred.
+        This function raises a `FunctionParameterTypeError` exception if one or more of its parameters have an incorrect type.
+        This function raises a `ValueError` exception if `append_value` parameter is empty (i.e
+        is equal to an empty string, an empty array or empty dictionary).
+        This function raises a `JSONPathError` exception if JSON path is not valid.
+        This function raises any additional exceptions if occurred.
 
         Examples:
 
@@ -222,9 +227,10 @@ class JsonFileParser:
         # Output: { "colors": [ "red", "blue", "green" ] }
 
 
-        For more information about this method, please visit: <LINK_TO_DOCUMENTATION_HERE>
+        For more information about this method please visit:
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#file-module-methods-and-properties
         """
-        # TODO Add link to an appropriate README section from GitHub.
+
         if type(json_path) != str:
             raise IncorrectFunctionParameterTypeError(
                 "json_path", "str", type(json_path).__name__
@@ -263,7 +269,7 @@ class JsonFileParser:
                                 i.update(append_value)
                             else:
                                 raise TypeError(
-                                    f"To append to a JSON array, parameter `append_value` must be a dictionary; got `{type(append_value).__name__}` instead."
+                                    f"To append to a JSON object, parameter `append_value` must be a dictionary; got `{type(append_value).__name__}` instead."
                                 )
                     self.active_json = json_content
                     return json_content
@@ -275,30 +281,31 @@ class JsonFileParser:
         self,
         json_path: str,
         key_or_index: Union[str, int],
-        new_value: any,
+        new_value: Any,
         strict_mode: bool = False,
     ) -> dict:
         """
         Update value in JSON.
 
         Parameters: `json_path : str` specifies property path, while `key_or_index : Union[str, int]`
-        specifies property name in JSON object or item index in JSON array. For example, if you
-        need to update value with key `update_key` located under `field1.field2.field3.update_key`,
+        specifies key in JSON object/item index in JSON array. For example, if you
+        need to update value with key `update_key` located under `field1.field2.field3.update_key`
         then parameter `key_or_index` will be equal to 'update_key' and `json_path` parameter will
-        be equal to `field1.field2.field3`. If you want to update value in root object, then `json_path`
-        parameter need to be equal to `$`. To update an item in an array, simply pass this item's
+        be equal to `field1.field2.field3`. If you want to update value in the root of the object then `json_path`
+        parameter needs to be equal to `$`. To update item in an array, simply pass this items'
         index (integer) as `key_or_index` parameter. Note: if you use an array index while `json_path`
-        parameter is pointing to a JSON object, or if you use a property name while `json_path` is pointing
-        to a JSON array, an exception will be raised. `new_value : any` specifies the new value that will overwrite
-        the old one. `strict_mode : bool` parameter enables strict mode. If set to `True`, this function will
-        compare the types of previous value and the new one. If they are not identical, an exception will be raised.
+        parameter is pointing to the JSON object or if you use a key name while `json_path` is pointing
+        to the JSON array, this function will raise an exception. `new_value : Any` specifies value that will overwrite
+        the old one. `strict_mode : bool` parameter enables Strict Mode. If set to `True`, this function will
+        compare the types of previous value and the new one. If they are not identical, this function will raise an exception.
 
-        This function will return a Python dictionary with updated content.
+        This function returns a Python dictionary with updated content.
 
-        This function will raise a `IncorrectFunctionParameterTypeError` exception if at least one of the
-        parameters has an incorrect type.
-        This function will raise a `JSONStrictModeError` if types of old and new values are not the same.
-        This function will raise any additional exceptions if occurred.
+        This function raises an `IncorrectFunctionParameterTypeError` exception if one or more of its' parameters
+        have incorrect types.
+        Tis function raises a `JSONPathError` if JSON path is not valid.
+        This function raises a `JSONStrictModeError` if types of old and new values are not the same while Strict Mode is enabled.
+        This function raises any additional exceptions if occurred.
 
         Examples:
 
@@ -353,7 +360,7 @@ class JsonFileParser:
         # Output: { "id": 1087, "name": "Jamie Kellen" }
 
         For more information about this method, please visit:
-        <LINK_TO_DOCUMENTATION_HERE>
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#file-module-methods-and-properties
         """
         # TODO Add link to an appropriate README section from GitHub
 
@@ -414,20 +421,19 @@ class JsonFileParser:
 
         Parameters: `json_path : str` specifies property path, while `key_or_index : Union[str, int]`
         specifies property name in JSON object or item index in JSON array. For example, if you
-        need to delete value with key `delete_key` located under `field1.field2.field3.delete_key`,
+        need to delete value with key `delete_key` located under `field1.field2.field3.delete_key`
         then parameter `key_or_index` will be equal to 'delete_key' and `json_path` parameter will
         be equal to `field1.field2.field3`.
-        To delete an item in an array, simply pass this item's
+        To delete an item in the array, simply pass this items'
         index (integer) as `key_or_index` parameter. Note: if you use an array index while `json_path`
-        parameter is pointing to a JSON object, or if you use a property name while `json_path` is pointing
-        to a JSON array, an exception will be raised.
+        parameter is pointing to a JSON object or if you use a property name while `json_path` is pointing
+        to a JSON array, this function will raise an exception.
 
-        This function will return a Python dictionary with updated content.
+        This function returns a Python dictionary with updated content.
 
-        This function will raise a `IncorrectFunctionParameterTypeError` exception if at least one of
-        given parameters has an incorrect type.
-        This function will raise a `JSONPathError` is given path is not valid.
-        This function will raise any additional exceptions if occurred.
+        This function raises an `IncorrectFunctionParameterTypeError` exception if one or more of its parameters have incorrect types.
+        This function raises a `JSONPathError` if JSON path is not valid.
+        This function raises any additional exceptions if occurred.
 
         Examples:
 
@@ -470,7 +476,7 @@ class JsonFileParser:
         # Output: { "colors": [ "magenta", "green" ] }
 
         For more information about this method, please visit:
-        <LINK_TO_DOCUMENTATION_HERE>
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#file-module-methods-and-properties
 
         """
 
@@ -517,9 +523,8 @@ class JsonFileParser:
         Minify all JSON in source file into one line.
 
         For more information about this method
-        please visit: <LINK_TO_DOCUMENTATION_HERE>
+        please visit: https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#file-module-methods-and-properties
         """
-        # TODO Add link to an appropriate README section from GitHub.
 
         file = open(self.__path, "r")
         unfiltered = file.read()
@@ -536,15 +541,14 @@ class JsonFileParser:
         and improve its readability.
 
         Parameters: `indent : int` specifies the number of spaces added.
-        If not provided, default value will be used (4 spaces/1 tab)
+        If not provided, this function will use the default value (4 spaces/1 tab)
 
-        This function will raise an `IncorrectFunctionParameterTypeError`
+        This function raises an `IncorrectFunctionParameterTypeError`
         exception if `indent` parameter has an incorrect type.
 
         For more information about this method please visit:
-        <LINK_TO_DOCUMENTATION_HERE>
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#file-module-methods-and-properties
         """
-        # TODO Add link to an appropriate README section from GitHub.
 
         if type(indent) != int and indent != None:
             raise IncorrectFunctionParameterTypeError(
@@ -569,16 +573,16 @@ class JsonFileParser:
         Discard changes to JSON.
 
         If called, this function will return initial JSON object
-        (without any changes)
+        (without any changes).
 
         Parameters: `discard_active_object : bool` specifies if
         changes will be discarded on active object or not. If
         set to `True`, active object will be reset to an initial
         state, otherwise it will be left untouched.
 
-        This function will return a Python dictionary with initial object.
+        This function returns a Python dictionary with initial object.
 
-        This function will raise an `IncorrectFunctionParameterTypeError`
+        This function raises an `IncorrectFunctionParameterTypeError`
         exception if `discard_active_object` has an incorrect type.
 
         Examples:
@@ -618,11 +622,9 @@ class JsonFileParser:
         # Please use this with extreme caution!
 
         For more information about this method, please visit:
-        <LINK_TO_DOCUMENTATION_HERE>
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#file-module-methods-and-properties
         """
 
-        # TODO Add link to an appropriate README section from GitHub.
-
         if type(discard_active_object) != bool:
             raise IncorrectFunctionParameterTypeError(
                 "discard_active_object", "bool", type(discard_active_object).__name__
@@ -637,12 +639,14 @@ class JsonFileParser:
         self, path: str = None, prettify: bool = True, create_file: bool = False
     ) -> None:
         """
-        Save JSON object to external file.
+        Save JSON object to file.
 
         Parameters: `path : str` specifies path to an external file. If not provided, object will be saved into source file.
         `prettify : bool` enables indentations in JSON (improves its readability). `create_file : bool` enables file creation.
-        If set to `True`, this function will create a new file and save JSON there, if provided path leads to non-existing file. Note:
-        if set to `True` and path is pointing to an existing file, an exception will be raised.
+        If set to `True`, this function will create a new file and save JSON there, if provided path leads to file that does not exist. Note:
+        if set to `True` and path is pointing to an existing file, this function will raise an exception.
+
+        This function raises an `IncorrectFunctionParameterTypeError` if one or more of its parameters have incorrect type.
 
         Examples:
 
@@ -692,11 +696,9 @@ class JsonFileParser:
         # Object from 'new_file.json' >> { "user_name": "Amasi0022", "date_of_registration": "19-07-2019", "role": "guest" }
 
         For more information about this method, please visit:
-        <LINK_TO_DOCUMENTATION_HERE>
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#file-module-methods-and-properties
         """
 
-        # TODO Add link to an appropriate README section from GitHub.
-
         if type(path) != str and path != None:
             raise IncorrectFunctionParameterTypeError(
                 "path", "str", type(path).__name__
@@ -715,7 +717,7 @@ class JsonFileParser:
         if path == None:
             file_path = self.__path
         else:
-            file_path == path
+            file_path = path
 
         file_json = self.active_json
 
@@ -739,4 +741,4 @@ class JsonFileParser:
                 )
             file = open(file_path, "w")
             file.write(JSON.dumps(file_json, indent=indent))
-            file.close()
\ No newline at end of file
+            file.close()
