@@ -32,10 +32,15 @@ from robust_json.errors import (
 )
 from robust_json.__internal_utils import service
 
-#! Review documentation!!!
-
 
 class JsonObjectParser:
+    """
+    This class provides functionality for working with JSON objects directly.
+
+    For more information please visit:
+    https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#object-module-overview
+    """
+
     def __init__(self, json: dict):
 
         if type(json) != dict:
@@ -56,17 +61,19 @@ class JsonObjectParser:
 
     def get_key_value(self, json_path: str) -> any:
         """
-        Retrieve specific value from JSON.
+        Retrieve specific key:value pair from JSON.
 
-        This function will fetch value from JSON object according to provided path.
+        This function fetches key:value pair from JSON object according to provided path
+        and returns only the value.
 
         Parameters: `json_path : str` specifies JSON property path (e.g. field1.field2.[...].fieldn).
 
-        This function will raise a `IncorrectFunctionParameterTypeError` if
+        This function raises an `IncorrectFunctionParameterTypeError` if
         `json_path` parameter has an incorrect type.
-        This function will raise a `JSONPathError` if given path is not valid.
+        This function raises a `JSONPathError` if JSON path is not valid.
+        This function raises any additional exceptions if occurred.
 
-        Example:
+        Examples:
 
         Retrieving key:value pair from a simple object:
 
@@ -87,9 +94,8 @@ class JsonObjectParser:
         # arr_val = 2
 
         For more information about this method please visit:
-        <LINK_TO_DOCUMENTATION_HERE>
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#object-module-methods-and-properties
         """
-        # TODO Add link to an appropriate README section from GitHub.
 
         if type(json_path) != str:
             raise IncorrectFunctionParameterTypeError(
@@ -115,76 +121,73 @@ class JsonObjectParser:
         """
         Append new value to an existing JSON object.
 
-        This function will take given value and add it to an existing JSON object.
+         This function takes value and adds it to the JSON object.
 
-        Parameters: `json_path : str` specifies JSON property path, where the given
-        value needs to be added. If there is a need to append a value
-        to the root object, this parameter needs to be equal to `$`. `append_value : any`
-        specifies the value that will be appended to the object. `append_at_end : bool`
-        specifies if given value will be appended to the end of a JSON array or it
-        will be added into each object of this array. While this parameter has no effect
-        if value is being appended to a JSON object (dictionary), it is neccessary if
-        value is being appended to a JSON array (list). If value is being appended to an array of elements,
-        other than arrays or objects (integers, strings, etc.), setting this parameter to `False` will not
-        result in any changes made to array. It needs to be set to `True` for this operation to complete.
+         Parameters: `json_path : str` specifies JSON property path where the given
+         value needs to be added. If there is a need to append a value
+         to the root of the object, this parameter needs to be equal to `$`. `append_value : Any`
+         specifies the value that will be appended to the object. `append_at_end : bool`
+         controls the behaviour of this function regarding JSON arrays of objects (structures like this: [{}, {}, {}, ...])
+         and general arrays (structures like this: [a, b, c, ...]). It has no influence on other structures. If set to False,
+         function will try to add value to each object of an array. If set to True, function will try to append
+         value at the end of an array. (see examples below).
 
 
-        This function will return a Python dictionary with updated content.
+         This function returns a Python dictionary with updated content.
 
-        This function will raise a `FunctionParameterTypeError` exception if either of given parameters
-        has an incorrect type.
-        This function will raise a `ValueError` exception if `append_value` parameter is empty (i.e
-        it's equal to an empty string, an empty array or empty dictionary).
-        This function will raise a `JSONPathError` exception if given path is not valid.
-        This function will raise any additional exceptions if occurred.
+         This function raises a `FunctionParameterTypeError` exception if one or more of its parameters have an incorrect type.
+         This function raises a `ValueError` exception if `append_value` parameter is empty (i.e
+         is equal to an empty string, an empty array or empty dictionary).
+         This function raises a `JSONPathError` exception if JSON path is not valid.
+         This function raises any additional exceptions if occurred.
 
-        Examples:
+         Examples:
 
-        Adding a simple key:value pair to the root object:
+         Adding a simple key:value pair to the root object:
 
-        >>> from robust_json.object import JsonObjectParser
-        >>> obj = { "key": "value" }
-        >>> op = JsonObjectParser(obj) # You can also pass dictionary directly
-        >>> op.append('$', { 'test': 'test' })
-        >>> op.active_json
-        # Output: { "key": "value", "test": "test" }
+         >>> from robust_json.object import JsonObjectParser
+         >>> obj = { "key": "value" }
+         >>> op = JsonObjectParser(obj) # You can also pass dictionary directly
+         >>> op.append('$', { 'test': 'test' })
+         >>> op.active_json
+         # Output: { "key": "value", "test": "test" }
 
-        Adding a new JSON object to an array of objects:
+         Adding a new JSON object to an array of objects:
 
-        >>> from robust_json.object import JsonObjectParser
-        >>> op = JsonObjectParser({ "users": [ {"id": 1, "name": "Ken"}, { "id": 2, "name": "Liza" } ] })
-        >>> op.append('users', { 'id': 3, 'name': 'Nick' }, True)
-        >>> op.active_json
-        # Output: { "users": [ {"id": 1, "name": "Ken"}, { "id": 2, "name": "Liza" }, { "id": 3, "name": "Nick" } ] }
+         >>> from robust_json.object import JsonObjectParser
+         >>> op = JsonObjectParser({ "users": [ {"id": 1, "name": "Ken"}, { "id": 2, "name": "Liza" } ] })
+         >>> op.append('users', { 'id': 3, 'name': 'Nick' }, True)
+         >>> op.active_json
+         # Output: { "users": [ {"id": 1, "name": "Ken"}, { "id": 2, "name": "Liza" }, { "id": 3, "name": "Nick" } ] }
 
-        Adding a key:value pair in each object of an array of objects
+         Adding a key:value pair in each object of an array of objects
 
-        >>> from robust_json.object import JsonObjectParser
-        >>> obj = { "users": [ {"id": 1, "name": "Ken"}, { "id": 2, "name": "Liza" } ] }
-        >>> op = JsonObjectParser(obj) # You can also pass dictionary directly
-        >>> op.append('users', { 'role': 'guest' })
-        >>> op.active_json
-        # Output: { "users": [ { "id": 1, "name": "Ken", "role": "guest" }, { "id": 2, "name": "Liza", "role": "guest" } ] }
+         >>> from robust_json.object import JsonObjectParser
+         >>> obj = { "users": [ {"id": 1, "name": "Ken"}, { "id": 2, "name": "Liza" } ] }
+         >>> op = JsonObjectParser(obj) # You can also pass dictionary directly
+         >>> op.append('users', { 'role': 'guest' })
+         >>> op.active_json
+         # Output: { "users": [ { "id": 1, "name": "Ken", "role": "guest" }, { "id": 2, "name": "Liza", "role": "guest" } ] }
 
-        Adding a new element to an array of strings:
+         Adding a new element to an array of strings:
 
-        >>> from robust_json.object import JsonObjectParser
-        >>> obj = { "colors": [ "red", "blue" ] }
-        >>> op = JsonObjectParser(obj)
-        >>> op.append('colors', 'green')
-        >>> op.active_json
-        # Output: { "colors": [ "red", "blue" ] }
-        # Nothing has appended. It's because this function tried to append given
-        # value to each string in array and failed
-        # To fix this, you need to set `append_at_end` parameter to `True`
-        >>> op.append('colors', 'green', True)
-        >>> op.active_json
-        # Output: { "colors": [ "red", "blue", "green" ] }
+         >>> from robust_json.object import JsonObjectParser
+         >>> obj = { "colors": [ "red", "blue" ] }
+         >>> op = JsonObjectParser(obj)
+         >>> op.append('colors', 'green')
+         >>> op.active_json
+         # Output: { "colors": [ "red", "blue" ] }
+         # Nothing has appended. It's because this function tried to append given
+         # value to each string in array and failed
+         # To fix this, you need to set `append_at_end` parameter to `True`
+         >>> op.append('colors', 'green', True)
+         >>> op.active_json
+         # Output: { "colors": [ "red", "blue", "green" ] }
 
 
-        For more information about this method, please visit: <LINK_TO_DOCUMENTATION_HERE>
+         For more information about this method, please visit: https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#object-module-methods-and-properties
         """
-        # TODO Add link to an appropriate README section from GitHub.
+
         if type(json_path) != str:
             raise IncorrectFunctionParameterTypeError(
                 "json_path", "str", type(json_path).__name__
@@ -242,23 +245,24 @@ class JsonObjectParser:
         Update value in JSON.
 
         Parameters: `json_path : str` specifies property path, while `key_or_index : Union[str, int]`
-        specifies property name in JSON object or item index in JSON array. For example, if you
-        need to update value with key `update_key` located under `field1.field2.field3.update_key`,
+        specifies key in JSON object/item index in JSON array. For example, if you
+        need to update value with key `update_key` located under `field1.field2.field3.update_key`
         then parameter `key_or_index` will be equal to 'update_key' and `json_path` parameter will
-        be equal to `field1.field2.field3`. If you want to update value in root object, then `json_path`
-        parameter need to be equal to `$`. To update an item in an array, simply pass this item's
+        be equal to `field1.field2.field3`. If you want to update value in the root of the object then `json_path`
+        parameter needs to be equal to `$`. To update item in an array, simply pass this items'
         index (integer) as `key_or_index` parameter. Note: if you use an array index while `json_path`
-        parameter is pointing to a JSON object, or if you use a property name while `json_path` is pointing
-        to a JSON array, an exception will be raised. `new_value : any` specifies the new value that will overwrite
-        the old one. `strict_mode : bool` parameter enables strict mode. If set to `True`, this function will
-        compare the types of previous value and the new one. If they are not identical, an exception will be raised.
+        parameter is pointing to the JSON object or if you use a key name while `json_path` is pointing
+        to the JSON array, this function will raise an exception. `new_value : Any` specifies value that will overwrite
+        the old one. `strict_mode : bool` parameter enables Strict Mode. If set to `True`, this function will
+        compare the types of previous value and the new one. If they are not identical, this function will raise an exception.
 
-        This function will return a Python dictionary with updated content.
+        This function returns a Python dictionary with updated content.
 
-        This function will raise a `IncorrectFunctionParameterTypeError` exception if at least one of the
-        parameters has an incorrect type.
-        This function will raise a `JSONStrictModeError` if types of old and new values are not the same.
-        This function will raise any additional exceptions if occurred.
+        This function raises an `IncorrectFunctionParameterTypeError` exception if one or more of its' parameters
+        have incorrect types.
+        Tis function raises a `JSONPathError` if JSON path is not valid.
+        This function raises a `JSONStrictModeError` if types of old and new values are not the same while Strict Mode is enabled.
+        This function raises any additional exceptions if occurred.
 
         Examples:
 
@@ -313,7 +317,7 @@ class JsonObjectParser:
         # Output: { "id": 1087, "name": "Jamie Kellen" }
 
         For more information about this method, please visit:
-        <LINK_TO_DOCUMENTATION_HERE>
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#object-module-methods-and-properties
         """
         # TODO Add link to an appropriate README section from GitHub
 
@@ -374,20 +378,19 @@ class JsonObjectParser:
 
         Parameters: `json_path : str` specifies property path, while `key_or_index : Union[str, int]`
         specifies property name in JSON object or item index in JSON array. For example, if you
-        need to delete value with key `delete_key` located under `field1.field2.field3.delete_key`,
+        need to delete value with key `delete_key` located under `field1.field2.field3.delete_key`
         then parameter `key_or_index` will be equal to 'delete_key' and `json_path` parameter will
         be equal to `field1.field2.field3`.
-        To delete an item in an array, simply pass this item's
+        To delete an item in the array, simply pass this items'
         index (integer) as `key_or_index` parameter. Note: if you use an array index while `json_path`
-        parameter is pointing to a JSON object, or if you use a property name while `json_path` is pointing
-        to a JSON array, an exception will be raised.
+        parameter is pointing to a JSON object or if you use a property name while `json_path` is pointing
+        to a JSON array, this function will raise an exception.
 
-        This function will return a Python dictionary with updated content.
+        This function returns a Python dictionary with updated content.
 
-        This function will raise a `IncorrectFunctionParameterTypeError` exception if at least one of
-        given parameters has an incorrect type.
-        This function will raise a `JSONPathError` is given path is not valid.
-        This function will raise any additional exceptions if occurred.
+        This function raises an `IncorrectFunctionParameterTypeError` exception if one or more of its parameters have incorrect types.
+        This function raises a `JSONPathError` if JSON path is not valid.
+        This function raises any additional exceptions if occurred.
 
         Examples:
 
@@ -430,7 +433,7 @@ class JsonObjectParser:
         # Output: { "colors": [ "magenta", "green" ] }
 
         For more information about this method, please visit:
-        <LINK_TO_DOCUMENTATION_HERE>
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#object-module-methods-and-properties
 
         """
 
@@ -477,17 +480,18 @@ class JsonObjectParser:
         Discard changes to JSON.
 
         If called, this function will return initial JSON object
-        (without any changes)
+        (without any changes).
 
         Parameters: `discard_active_object : bool` specifies if
         changes will be discarded on active object or not. If
         set to `True`, active object will be reset to an initial
-        state, otherwise it will be lef untouched.
+        state, otherwise it will be left untouched.
 
-        This function will return a Python dictionary with initial object.
+        This function returns a Python dictionary with initial object.
 
-        This function will raise an `IncorrectFunctionParameterTypeError`
+        This function raises an `IncorrectFunctionParameterTypeError`
         exception if `discard_active_object` has an incorrect type.
+        This function raises any additional exceptions if occurred.
 
         Examples:
 
@@ -526,11 +530,9 @@ class JsonObjectParser:
         # Please use this with extreme caution!
 
         For more information about this method, please visit:
-        <LINK_TO_DOCUMENTATION_HERE>
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#object-module-methods-and-properties
         """
 
-        # TODO Add link to an appropriate README section from GitHub.
-
         if type(discard_active_object) != bool:
             raise IncorrectFunctionParameterTypeError(
                 "discard_active_object", "bool", type(discard_active_object).__name__
@@ -545,12 +547,15 @@ class JsonObjectParser:
         self, path: str, prettify: bool = True, create_file: bool = False
     ) -> None:
         """
-        Save JSON object to external file.
+        Save JSON object to file.
 
         Parameters: `path : str` specifies path to an external file.
         `prettify : bool` enables indentations in JSON (improves its readability). `create_file : bool` enables file creation.
-        If set to `True`, this function will create a new file and save JSON there, if provided path leads to non-existing file. Note:
-        if set to `True` and path is pointing to an existing file, an exception will be raised.
+        If set to `True`, this function will create a new file and save JSON there, if provided path leads to file that does not exist. Note:
+        if set to `True` and path is pointing to an existing file, this function will raise an exception.
+
+        This function raises an `IncorrectFunctionParameterTypeError` if one or more of its parameters have incorrect type.
+        This function raises any additional exceptions if occurred.
 
         Examples:
 
@@ -587,11 +592,9 @@ class JsonObjectParser:
         # Object from `new_file.json` >> { "user_name": "Amasi0022", "date_of_registration": "19-07-2019", "role": "guest" }
 
         For more information about this method, please visit:
-        <LINK_TO_DOCUMENTATION_HERE>
+        https://github.com/NickolaiBeloguzov/robust-json/blob/master/README.md#object-module-methods-and-properties
         """
 
-        # TODO Add link to an appropriate README section from GitHub.
-
         if type(path) != str:
             raise IncorrectFunctionParameterTypeError(
                 "path", "str", type(path).__name__
@@ -631,4 +634,4 @@ class JsonObjectParser:
                 )
             file = open(file_path, "w")
             file.write(JSON.dumps(file_json, indent=indent))
-            file.close()
\ No newline at end of file
+            file.close()
